import random
from unittest.mock import call, patch

import responses
from django.test import override_settings
from django.urls import reverse
from rest_framework import status

from arkindex.images.models import Image, ImageServer
from arkindex.ponos.models import Farm
from arkindex.process.models import Process, ProcessMode
from arkindex.project.aws import S3FileStatus
from arkindex.project.tests import FixtureAPITestCase
from arkindex.users.models import Scope


class ImageMethods:
    def check_mock(self, save=True):
        self.path = "image_path"
        self.height = 1337
        self.width = 4200
        self.status = S3FileStatus.Checked
        if save:
            self.save()


@override_settings(LOCAL_IMAGESERVER_ID=1)
class TestImageApi(FixtureAPITestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.server = ImageServer.objects.create(url="https://test-iiif-server.teklia.com/images")

    def setUp(self):
        super().setUp()
        # A random 32-character hex string
        self.image_hash = f"{random.randrange(16**32):032x}"

    def test_create_image_requires_login(self):
        with self.assertNumQueries(0):
            response = self.client.post(reverse("api:image-create"), {"hash": self.image_hash})
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_create_image_requires_scope(self):
        self.assertFalse(self.user.user_scopes.filter(scope=Scope.UploadS3Image).exists())
        self.client.force_login(self.user)

        with self.assertNumQueries(3):
            response = self.client.post(reverse("api:image-create"), {"hash": self.image_hash})
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertDictEqual(response.json(), {
            "detail": "You do not have permission to perform this action.",
            "missing_scopes": [Scope.UploadS3Image.value],
        })

        self.user.user_scopes.create(scope=Scope.UploadS3Image)
        response = self.client.post(reverse("api:image-create"), {"hash": self.image_hash})
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_create_image_admin_noscope(self):
        self.assertFalse(self.superuser.user_scopes.filter(scope=Scope.UploadS3Image).exists())
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.post(reverse("api:image-create"), {"hash": self.image_hash})
            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch("arkindex.project.aws.s3.meta.client.generate_presigned_url")
    def test_create_image(self, s3_presigned_url_mock):
        """
        Use serializer to create a new Image
        Endpoint should return a s3 URI to upload image file
        """
        s3_presigned_url_mock.side_effect = ["http://s3/url", "http://s3/upload_put_url"]
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.post(reverse("api:image-create"), {
                "hash": self.image_hash,
                "path": "a/b/c",
            })
            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        data = response.json()
        self.assertIn("id", data)
        im = Image.objects.get(id=data["id"])
        self.assertEqual(im.path, "a%2Fb%2Fc")
        self.assertEqual(im.width, 0)
        self.assertEqual(im.height, 0)
        self.assertEqual(im.url, "http://server/a%2Fb%2Fc")
        self.assertEqual(im.status, S3FileStatus.Unchecked)
        self.assertEqual(im.hash, self.image_hash)
        self.assertDictEqual(
            response.json(),
            {
                "id": str(im.id),
                "path": "a%2Fb%2Fc",
                "width": 0,
                "height": 0,
                "url": "http://server/a%2Fb%2Fc",
                "status": "unchecked",
                "hash": self.image_hash,
                "s3_put_url": "http://s3/upload_put_url",
                "s3_url": "http://s3/url",
            }
        )

    @patch("arkindex.project.aws.s3.meta.client.generate_presigned_url")
    def test_create_image_default_path(self, s3_presigned_url_mock):
        """
        Use serializer to create a new Image
        Endpoint should return a s3 URI to upload image file
        """
        s3_presigned_url_mock.side_effect = ["http://s3/url", "http://s3/upload_put_url"]
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.post(reverse("api:image-create"), {"hash": self.image_hash})
            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        data = response.json()
        self.assertIn("id", data)
        im = Image.objects.get(id=data["id"])
        # Path should default to the image's UUID when not sent with the request
        self.assertEqual(im.path, str(im.id))
        self.assertEqual(im.width, 0)
        self.assertEqual(im.height, 0)
        self.assertEqual(im.url, f"http://server/{im.id}")
        self.assertEqual(im.status, S3FileStatus.Unchecked)
        self.assertEqual(im.hash, self.image_hash)
        self.assertDictEqual(
            response.json(),
            {
                "id": str(im.id),
                "path": str(im.id),
                "width": 0,
                "height": 0,
                "url": f"http://server/{im.id}",
                "status": "unchecked",
                "hash": self.image_hash,
                "s3_put_url": "http://s3/upload_put_url",
                "s3_url": "http://s3/url",
            }
        )

    def test_create_image_unique_hash(self):
        self.client.force_login(self.superuser)
        existing_image = self.imgsrv.images.create(path="something", hash=self.image_hash)

        with self.assertNumQueries(4):
            response = self.client.post(reverse("api:image-create"), {"hash": self.image_hash})
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        data = response.json()
        self.assertDictEqual(data, {
            "hash": ["Image with this hash already exists"],
            "id": str(existing_image.id),
        })

        self.assertEqual(Image.objects.filter(hash=self.image_hash).count(), 1)

    def test_create_image_unique_path(self):
        self.client.force_login(self.superuser)
        existing_image = self.imgsrv.images.create(path="something", hash=self.image_hash)
        new_hash = f"{random.randrange(16**32):032x}"

        with self.assertNumQueries(4):
            response = self.client.post(reverse("api:image-create"), {"hash": new_hash, "path": "something"})
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        data = response.json()
        self.assertDictEqual(data, {
            "path": ["Image with this path already exists"],
            "id": str(existing_image.id),
        })

        self.assertEqual(Image.objects.filter(path="something").count(), 1)

    @patch("arkindex.project.aws.s3.meta.client.generate_presigned_url")
    def test_create_image_ignore_attributes(self, presigned_url_mock):
        presigned_url_mock.side_effect = ["http://s3/url", "http://s3/put_url"]
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.post(reverse("api:image-create"), {
                "id": "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
                "path": "lol",
                "hash": self.image_hash,
                "width": 1234,
                "height": 1234,
                "url": "http://dev/null",
                "status": "checked",
                "s3_url": "http://a/b/c",
                "s3_put_url": "http://d/e/f",
                "server": 9999,
            })
            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        data = response.json()
        self.assertIn("id", data)
        image = Image.objects.get(id=data["id"])

        self.assertNotEqual(str(image.id), "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa")
        # Only the hash and path were set from the payload
        self.assertEqual(image.hash, self.image_hash)
        self.assertEqual(image.path, "lol")
        # Other attributes are using the default values
        self.assertEqual(image.width, 0)
        self.assertEqual(image.height, 0)
        self.assertEqual(image.status, S3FileStatus.Unchecked)
        self.assertEqual(image.server_id, self.imgsrv.id)

        self.assertDictEqual(
            response.json(),
            {
                "id": str(image.id),
                "path": "lol",
                "width": 0,
                "height": 0,
                "url": "http://server/lol",
                "status": "unchecked",
                "hash": self.image_hash,
                "s3_put_url": "http://s3/put_url",
                "s3_url": "http://s3/url",
            }
        )

    def test_partial_update_image_requires_login(self):
        img = ImageServer.objects.local.images.create(
            status=S3FileStatus.Unchecked,
            path="test_partial_update",
        )

        with self.assertNumQueries(0):
            response = self.client.patch(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        img.refresh_from_db()
        self.assertEqual(img.server, ImageServer.objects.local)
        self.assertEqual(img.path, "test_partial_update")
        self.assertEqual(img.status, S3FileStatus.Unchecked)
        self.assertEqual(img.width, 0)
        self.assertEqual(img.height, 0)

    @patch("arkindex.images.serializers.Image.perform_check")
    @patch("arkindex.images.serializers.Image.check_hash")
    def test_partial_update_image_check(self, check_hash_mock, perform_check_mock):
        """
        Test setting an image's status to Checked runs the image checks
        """
        img = ImageServer.objects.local.images.create(
            status=S3FileStatus.Unchecked,
            path="test_partial_update",
            width=1000,
            height=1000,
        )
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.patch(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_200_OK)

        img.refresh_from_db()
        self.assertEqual(img.status, S3FileStatus.Checked)
        self.assertEqual(img.server, ImageServer.objects.local)
        self.assertEqual(img.path, "test_partial_update")
        self.assertEqual(img.width, 1000)
        self.assertEqual(img.height, 1000)
        self.assertEqual(check_hash_mock.call_count, 1)
        self.assertEqual(perform_check_mock.call_count, 1)
        self.assertEqual(check_hash_mock.call_args, call(raise_exc=True, save=False))
        self.assertEqual(perform_check_mock.call_args, call(raise_exc=True, save=False))

    @responses.activate
    def test_partial_update_image_check_fail(self):
        """
        Test setting an image's status to Checked runs the image checks
        """
        url = "https://test-iiif-server.teklia.com/images/notfound/info.json"
        responses.add(responses.GET, url, status=404)
        img = self.server.images.create(
            status=S3FileStatus.Unchecked,
            path="notfound",
        )
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.patch(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(response.json(), {"status": [f"404 Client Error: Not Found for url: {url}"]})
        img.refresh_from_db()
        self.assertEqual(img.status, S3FileStatus.Error)
        self.assertEqual(img.server, self.server)
        self.assertEqual(img.path, "notfound")
        self.assertEqual(img.width, 0)
        self.assertEqual(img.height, 0)

    @patch("arkindex.images.serializers.Image.perform_check")
    @patch("arkindex.images.serializers.Image.check_hash")
    def test_partial_update_image_server_read_only(self, check_hash_mock, perform_check_mock):
        self.client.force_login(self.superuser)
        server = ImageServer.objects.create(
            display_name="S3 test",
            s3_bucket="s3_blanket",
            s3_read_only_bucket=True,
            url="http://some.server/",
        )
        img = Image.objects.create(
            server=server,
            status=S3FileStatus.Unchecked,
            path="test_partial_update",
            width=1000,
            height=1000,
        )

        with self.assertNumQueries(5):
            response = self.client.patch(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_200_OK)

        img.refresh_from_db()
        self.assertEqual(img.status, S3FileStatus.Checked)
        self.assertEqual(img.server, server)
        self.assertEqual(img.path, "test_partial_update")
        self.assertEqual(img.width, 1000)
        self.assertEqual(img.height, 1000)
        self.assertEqual(check_hash_mock.call_count, 0)
        self.assertEqual(perform_check_mock.call_count, 1)
        self.assertEqual(perform_check_mock.call_args, call(raise_exc=True, save=False))

    def test_update_image_requires_login(self):
        img = ImageServer.objects.local.images.create(
            status=S3FileStatus.Unchecked,
            path="test_update",
        )

        with self.assertNumQueries(0):
            response = self.client.put(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("arkindex.images.serializers.Image.perform_check")
    @patch("arkindex.images.serializers.Image.check_hash")
    def test_update_image_check(self, check_hash_mock, perform_check_mock):
        """
        Test setting an image's status to Checked runs the image checks
        """
        img = ImageServer.objects.local.images.create(
            status=S3FileStatus.Unchecked,
            path="test_update",
            width=1000,
            height=1000,
        )
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.put(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_200_OK)

        img.refresh_from_db()
        self.assertEqual(img.status, S3FileStatus.Checked)
        self.assertEqual(img.server, ImageServer.objects.local)
        self.assertEqual(img.path, "test_update")
        self.assertEqual(img.width, 1000)
        self.assertEqual(img.height, 1000)
        self.assertEqual(check_hash_mock.call_count, 1)
        self.assertEqual(perform_check_mock.call_count, 1)
        self.assertEqual(check_hash_mock.call_args, call(raise_exc=True, save=False))
        self.assertEqual(perform_check_mock.call_args, call(raise_exc=True, save=False))

    @responses.activate
    def test_update_image_check_fail(self):
        """
        Test setting an image's status to Checked runs the image checks
        """
        url = "https://test-iiif-server.teklia.com/images/notfound/info.json"
        responses.add(responses.GET, url, status=404)
        img = self.server.images.create(
            status=S3FileStatus.Unchecked,
            path="notfound",
        )
        self.client.force_login(self.superuser)

        with self.assertNumQueries(5):
            response = self.client.put(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(response.json(), {"status": [f"404 Client Error: Not Found for url: {url}"]})
        img.refresh_from_db()
        self.assertEqual(img.status, S3FileStatus.Error)
        self.assertEqual(img.server, self.server)
        self.assertEqual(img.path, "notfound")
        self.assertEqual(img.width, 0)
        self.assertEqual(img.height, 0)

    @patch("arkindex.images.serializers.Image.perform_check")
    @patch("arkindex.images.serializers.Image.check_hash")
    def test_update_image_server_read_only(self, check_hash_mock, perform_check_mock):
        self.client.force_login(self.superuser)
        server = ImageServer.objects.create(
            display_name="S3 test",
            s3_bucket="s3_blanket",
            s3_read_only_bucket=True,
            url="http://some.server/",
        )
        img = Image.objects.create(
            server=server,
            status=S3FileStatus.Unchecked,
            path="test_update",
            width=1000,
            height=1000,
        )

        with self.assertNumQueries(5):
            response = self.client.put(
                reverse("api:image-retrieve", kwargs={"pk": str(img.id)}),
                {"status": "checked"},
            )
            self.assertEqual(response.status_code, status.HTTP_200_OK)

        img.refresh_from_db()
        self.assertEqual(img.status, S3FileStatus.Checked)
        self.assertEqual(img.server, server)
        self.assertEqual(img.path, "test_update")
        self.assertEqual(img.width, 1000)
        self.assertEqual(img.height, 1000)
        self.assertEqual(check_hash_mock.call_count, 0)
        self.assertEqual(perform_check_mock.call_count, 1)
        self.assertEqual(perform_check_mock.call_args, call(raise_exc=True, save=False))

    @patch("arkindex.images.serializers.Image.perform_check", new=ImageMethods.check_mock)
    def test_create_iiif_image(self):
        self.client.force_login(self.superuser)
        response = self.client.post(
            reverse("api:iiif-url-create"),
            {"url": "https://test-iiif-server.teklia.com/images/image_path"}
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        data = response.json()
        image = Image.objects.get(id=data["id"])
        self.assertDictEqual(data, {
            "id": str(image.id),
            "status": "checked",
            "height": 1337,
            "width": 4200,
            "path": "image_path",
            "url": "https://test-iiif-server.teklia.com/images/image_path",
            "s3_url": None,
            "server": {
                "display_name": self.server.display_name,
                "max_height": self.server.max_height,
                "max_width": self.server.max_width,
                "url": "https://test-iiif-server.teklia.com/images"
            }
        })

    def test_create_iiif_image_requires_login(self):
        response = self.client.post(
            reverse("api:iiif-url-create"),
            {"url": "https://test-iiif-server.teklia.com/images/image_path"}
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertDictEqual(response.json(), {
            "detail": "Authentication credentials were not provided."
        })

    def test_create_iiif_image_scope(self):
        self.assertFalse(self.user.user_scopes.filter(scope=Scope.CreateIIIFImage).exists())
        self.client.force_login(self.user)
        response = self.client.post(
            reverse("api:iiif-url-create"),
            {"url": "https://test-iiif-server.teklia.com/images/image_path"}
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertDictEqual(response.json(), {
            "detail": "You do not have permission to perform this action.",
            "missing_scopes": [Scope.CreateIIIFImage.value],
        })

        self.user.user_scopes.create(scope=Scope.CreateIIIFImage)
        response = self.client.post(
            reverse("api:iiif-url-create"),
            {"url": "https://test-iiif-server.teklia.com/images/image_path"}
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertTrue(Image.objects.filter(path="image_path").exists())

    def test_create_iiif_image_admin_noscope(self):
        self.client.force_login(self.superuser)
        self.assertFalse(self.superuser.user_scopes.filter(scope=Scope.CreateIIIFImage).exists())
        response = self.client.post(
            reverse("api:iiif-url-create"),
            {"url": "https://test-iiif-server.teklia.com/images/image_path"}
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_create_iiif_image_new_server(self):
        self.client.force_login(self.superuser)
        self.assertFalse(ImageServer.objects.filter(url="https://randomserver.teklia.com/iiif").exists())
        response = self.client.post(
            reverse("api:iiif-url-create"),
            {"url": "https://randomserver.teklia.com/iiif/path/to/image.jpg"}
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertTrue(ImageServer.objects.filter(url="https://randomserver.teklia.com/iiif").exists())
        self.assertTrue(Image.objects.filter(path="path/to/image.jpg").exists())

    def test_create_iiif_image_wrong_urls(self):
        self.client.force_login(self.superuser)
        # Create a server with same prefix as existing one
        ImageServer.objects.create(url="https://test-iiif-server")
        wrong_urls = {
            "This field may not be blank.": [
                ""
            ],
            "Enter a valid URL.": [
                "1337",
                "images.com/iiif/007",
                "sftp://images.com/iiif/007"
            ],
            'The URL "https://not-a-server.teklia.com/v2/007" does not match any existing image server. '
            "Please ask an instance administrator to register the IIIF server for this image.": [
                "https://not-a-server.teklia.com/v2/007",
            ],
            'The URL "https://test-iiif-server.teklia.com/images/v2/images/image.jpg" matched multiple existing image servers': [
                "https://test-iiif-server.teklia.com/images/v2/images/image.jpg"
            ]
        }
        for message, urls in wrong_urls.items():
            for url in urls:
                response = self.client.post(reverse("api:iiif-url-create"), {"url": url})
                self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
                self.assertEqual(response.json(), {"url": [message]})

    def test_create_existing_iiif_image(self):
        self.client.force_login(self.superuser)
        api_call = (
            reverse("api:iiif-url-create"),
            {"url": "https://test-iiif-server.teklia.com/images/image_path"}
        )
        response = self.client.post(*api_call)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        created_image_id = response.json()["id"]
        response = self.client.post(*api_call)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertDictEqual(response.json(), {
            "__all__": ["Image with this path already exists"],
            "id": created_image_id
        })

    @patch("arkindex.images.serializers.Image.perform_check", new=ImageMethods.check_mock)
    def test_create_existing_iiif_image_check(self):
        """
        Special case when an image does not look duplicated,
        except the check algorithm updates its path and turns it into a duplicate
        """
        self.client.force_login(self.superuser)
        img = self.server.images.create(
            status=S3FileStatus.Unchecked,
            path="image_path",
        )
        response = self.client.post(
            reverse("api:iiif-url-create"),
            {"url": "https://test-iiif-server.teklia.com/images/image_path2"},
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertDictEqual(response.json(), {
            "__all__": ["Image with this path already exists"],
            "id": str(img.id),
        })

    def test_create_image_information_requires_login(self):
        response = self.client.post(
            reverse("api:iiif-info-create"),
            {
                "@id": "https://test-iiif-server.teklia.com/images/image_path",
                "height": 1337,
                "width": 4200,
                "profile": ["http://iiif.io/api/image/2/level2.json"],
            },
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_create_image_information_ponos_task(self):
        """
        IIIF images created by a Ponos task are immediately checked
        """
        process = Process.objects.create(
            mode=ProcessMode.Workers,
            creator=self.user,
            corpus=self.corpus,
            farm=Farm.objects.first(),
        )
        process.run()
        task = process.tasks.first()
        # The user scope should not be necessary with Ponos task authentication
        self.assertFalse(self.user.user_scopes.filter(scope=Scope.CreateIIIFImage).exists())

        response = self.client.post(
            reverse("api:iiif-info-create"),
            {
                "@id": "https://test-iiif-server.teklia.com/images/image_path",
                "height": 1337,
                "width": 4200,
                "profile": ["http://iiif.io/api/image/2/level2.json"],
            },
            HTTP_AUTHORIZATION=f"Ponos {task.token}",
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        data = response.json()
        image = Image.objects.get(id=data["id"])
        self.assertDictEqual(data, {
            "id": str(image.id),
            "status": "checked",
            "height": 1337,
            "width": 4200,
            "path": "image_path",
            "url": "https://test-iiif-server.teklia.com/images/image_path",
            "s3_url": None,
            "server": {
                "display_name": self.server.display_name,
                "max_height": self.server.max_height,
                "max_width": self.server.max_width,
                "url": "https://test-iiif-server.teklia.com/images"
            }
        })

    def test_create_image_information_existing(self):
        """Same test as above but when the image exists in DB"""
        self.server.images.create(
            path="image_path",
            width=200,
            height=100,
        )
        process = Process.objects.create(
            mode=ProcessMode.Workers,
            creator=self.user,
            corpus=self.corpus,
            farm=Farm.objects.first(),
        )
        process.run()
        task = process.tasks.first()
        # The user scope should not be necessary with Ponos task authentication
        self.assertFalse(self.user.user_scopes.filter(scope=Scope.CreateIIIFImage).exists())

        response = self.client.post(
            reverse("api:iiif-info-create"),
            {
                "@id": "https://test-iiif-server.teklia.com/images/image_path",
                "height": 1337,
                "width": 4200,
                "profile": ["http://iiif.io/api/image/2/level2.json"],
            },
            HTTP_AUTHORIZATION=f"Ponos {task.token}",
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        data = response.json()
        image = Image.objects.get(id=data["id"])
        self.assertDictEqual(data, {
            "id": str(image.id),
            "status": "checked",
            "height": 1337,
            "width": 4200,
            "path": "image_path",
            "url": "https://test-iiif-server.teklia.com/images/image_path",
            "s3_url": None,
            "server": {
                "display_name": self.server.display_name,
                "max_height": self.server.max_height,
                "max_width": self.server.max_width,
                "url": "https://test-iiif-server.teklia.com/images"
            }
        })

    def test_create_image_information_external_scope(self):
        self.assertFalse(self.user.user_scopes.filter(scope=Scope.CreateIIIFImage).exists())
        self.client.force_login(self.user)
        payload = {
            "@id": "https://test-iiif-server.teklia.com/images/image_path",
            "height": 1337,
            "width": 4200,
            "profile": ["http://iiif.io/api/image/2/level2.json"],
        }
        response = self.client.post(reverse("api:iiif-info-create"), payload)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.user.user_scopes.create(scope=Scope.CreateIIIFImage)
        response = self.client.post(reverse("api:iiif-info-create"), payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        data = response.json()
        image = Image.objects.get(id=data["id"])
        # A successful image check from non-internal users should cause the image to stay unchecked
        self.assertEqual(image.status, S3FileStatus.Unchecked)

    def test_create_image_information_bad_content(self):
        self.client.force_login(self.superuser)
        response = self.client.post(reverse("api:iiif-info-create"), [], format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertDictEqual(response.json(), {"detail": ["Payload should be a JSON object"]})

        response = self.client.post(reverse("api:iiif-info-create"), {})
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertDictEqual(response.json(), {"@id": ["Image identifier is required"]})

    def test_create_image_information_server_error(self):
        self.client.force_login(self.superuser)
        response = self.client.post(
            reverse("api:iiif-info-create"),
            {
                "@id": "https://toast-server.teklia.com/images/image_path",
                "height": 1337,
                "width": 4200,
                "profile": ["http://iiif.io/api/image/2/level2.json"],
            },
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertDictEqual(response.json(), {"@id": [
            'The URL "https://toast-server.teklia.com/images/image_path" does not match any existing image server. '
            "Please ask an instance administrator to register the IIIF server for this image."
        ]})

    def test_create_image_information_check_error(self):
        self.client.force_login(self.superuser)
        response = self.client.post(
            reverse("api:iiif-info-create"),
            {
                "@id": "https://test-iiif-server.teklia.com/images/image_path",
                "height": 0,
                "width": 0,
                "profile": ["http://iiif.io/api/image/2/level2.json"],
            },
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        image = self.server.images.get(path="image_path")
        self.assertEqual(image.status, S3FileStatus.Error)
        self.assertDictEqual(response.json(), {
            "status": ["Image dimensions must be strictly positive integers"],
            "id": str(image.id),
        })
